@@ -1,5 +1,5 @@
 """
-Like described in the :mod:`jedi.parser.python.tree` module,
+Like described in the :mod:`parso.python.tree` module,
 there's a need for an ast like module to represent the states of parsed
 modules.
 
@@ -32,6 +32,8 @@ py__package__()                        Only on modules. For the import system.
 py__path__()                           Only on modules. For the import system.
 py__get__(call_object)                 Only on instances. Simulates
                                        descriptors.
+py__doc__(include_call_signature:      Returns the docstring for a context.
+          bool)
 ====================================== ========================================
 
 """
@@ -41,11 +43,12 @@ import imp
 import re
 from itertools import chain
 
+from parso.python import tree
+from parso import python_bytes_to_unicode
+
 from jedi._compatibility import use_metaclass
-from jedi.parser.python import tree
 from jedi import debug
-from jedi import common
-from jedi.evaluate.cache import memoize_default, CachedMetaClass, NO_DEFAULT
+from jedi.evaluate.cache import evaluator_method_cache, CachedMetaClass
 from jedi.evaluate import compiled
 from jedi.evaluate import recursion
 from jedi.evaluate import iterable
@@ -59,9 +62,10 @@ from jedi.evaluate.filters import ParserTreeFilter, FunctionExecutionFilter, \
     GlobalNameFilter, DictFilter, ContextName, AbstractNameDefinition, \
     ParamName, AnonymousInstanceParamName, TreeNameDefinition, \
     ContextNameMixin
-from jedi.evaluate.dynamic import search_params
 from jedi.evaluate import context
 from jedi.evaluate.context import ContextualizedNode
+from jedi import parser_utils
+from jedi.evaluate.parser_cache import get_yield_exprs
 
 
 def apply_py__get__(context, base_context):
@@ -111,7 +115,7 @@ class ClassContext(use_metaclass(CachedMetaClass, context.TreeContext)):
         super(ClassContext, self).__init__(evaluator, parent_context=parent_context)
         self.tree_node = classdef
 
-    @memoize_default(default=())
+    @evaluator_method_cache(default=())
     def py__mro__(self):
         def add(cls):
             if cls not in mro:
@@ -147,7 +151,7 @@ class ClassContext(use_metaclass(CachedMetaClass, context.TreeContext)):
                         add(cls_new)
         return tuple(mro)
 
-    @memoize_default(default=())
+    @evaluator_method_cache(default=())
     def py__bases__(self):
         arglist = self.tree_node.get_super_arglist()
         if arglist:
@@ -166,7 +170,7 @@ class ClassContext(use_metaclass(CachedMetaClass, context.TreeContext)):
     def get_params(self):
         from jedi.evaluate.instance import AnonymousInstance
         anon = AnonymousInstance(self.evaluator, self.parent_context, self)
-        return [AnonymousInstanceParamName(anon, param.name) for param in self.funcdef.params]
+        return [AnonymousInstanceParamName(anon, param.name) for param in self.funcdef.get_params()]
 
     def get_filters(self, search_global, until_position=None, origin_scope=None, is_instance=False):
         if search_global:
@@ -189,14 +193,6 @@ class ClassContext(use_metaclass(CachedMetaClass, context.TreeContext)):
     def is_class(self):
         return True
 
-    def get_subscope_by_name(self, name):
-        raise DeprecationWarning
-        for s in self.py__mro__():
-            for sub in reversed(s.subscopes):
-                if sub.name.value == name:
-                    return sub
-        raise KeyError("Couldn't find subscope.")
-
     def get_function_slot_names(self, name):
         for filter in self.get_filters(search_global=False):
             names = filter.get(name)
@@ -220,6 +216,20 @@ class ClassContext(use_metaclass(CachedMetaClass, context.TreeContext)):
         return ContextName(self, self.tree_node.name)
 
 
+class LambdaName(AbstractNameDefinition):
+    string_name = '<lambda>'
+
+    def __init__(self, lambda_context):
+        self._lambda_context = lambda_context
+        self.parent_context = lambda_context.parent_context
+
+    def start_pos(self):
+        return self._lambda_context.tree_node.start_pos
+
+    def infer(self):
+        return set([self._lambda_context])
+
+
 class FunctionContext(use_metaclass(CachedMetaClass, context.TreeContext)):
     """
     Needed because of decorators. Decorators are evaluated here.
@@ -248,17 +258,17 @@ class FunctionContext(use_metaclass(CachedMetaClass, context.TreeContext)):
         """
         Created to be used by inheritance.
         """
-        if self.tree_node.is_generator():
+        yield_exprs = get_yield_exprs(self.evaluator, self.tree_node)
+        if yield_exprs:
             return set([iterable.Generator(self.evaluator, function_execution)])
         else:
             return function_execution.get_return_values()
 
     def get_function_execution(self, arguments=None):
-        e = self.evaluator
         if arguments is None:
-            return AnonymousFunctionExecution(e, self.parent_context, self)
-        else:
-            return FunctionExecutionContext(e, self.parent_context, self, arguments)
+            arguments = param.AnonymousArguments()
+
+        return FunctionExecutionContext(self.evaluator, self.parent_context, self, arguments)
 
     def py__call__(self, arguments):
         function_execution = self.get_function_execution(arguments)
@@ -267,7 +277,7 @@ class FunctionContext(use_metaclass(CachedMetaClass, context.TreeContext)):
     def py__class__(self):
         # This differentiation is only necessary for Python2. Python3 does not
         # use a different method class.
-        if isinstance(self.tree_node.get_parent_scope(), tree.Class):
+        if isinstance(parser_utils.get_parent_scope(self.tree_node), tree.Class):
             name = 'METHOD_CLASS'
         else:
             name = 'FUNCTION_CLASS'
@@ -275,11 +285,14 @@ class FunctionContext(use_metaclass(CachedMetaClass, context.TreeContext)):
 
     @property
     def name(self):
+        if self.tree_node.type == 'lambdef':
+            return LambdaName(self)
         return ContextName(self, self.tree_node.name)
 
     def get_param_names(self):
         function_execution = self.get_function_execution()
-        return [ParamName(function_execution, param.name) for param in self.tree_node.params]
+        return [ParamName(function_execution, param.name)
+                for param in self.tree_node.get_params()]
 
 
 class FunctionExecutionContext(context.TreeContext):
@@ -299,20 +312,20 @@ class FunctionExecutionContext(context.TreeContext):
         self.tree_node = function_context.tree_node
         self.var_args = var_args
 
-    @memoize_default(default=set())
+    @evaluator_method_cache(default=set())
     @recursion.execution_recursion_decorator()
     def get_return_values(self, check_yields=False):
         funcdef = self.tree_node
-        if funcdef.type == 'lambda':
+        if funcdef.type == 'lambdef':
             return self.evaluator.eval_element(self, funcdef.children[-1])
 
         if check_yields:
             types = set()
-            returns = funcdef.yields
+            returns = get_yield_exprs(self.evaluator, funcdef)
         else:
-            returns = funcdef.returns
-            types = set(docstrings.find_return_types(self.get_root_context(), funcdef))
-            types |= set(pep0484.find_return_types(self.get_root_context(), funcdef))
+            returns = funcdef.iter_return_stmts()
+            types = set(docstrings.infer_return_types(self.function_context))
+            types |= set(pep0484.infer_return_types(self.function_context))
 
         for r in returns:
             check = flow_analysis.reachability_check(self, funcdef, r)
@@ -322,13 +335,23 @@ class FunctionExecutionContext(context.TreeContext):
                 if check_yields:
                     types |= set(self._eval_yield(r))
                 else:
-                    types |= self.eval_node(r.children[1])
+                    try:
+                        children = r.children
+                    except AttributeError:
+                        types.add(compiled.create(self.evaluator, None))
+                    else:
+                        types |= self.eval_node(children[1])
             if check is flow_analysis.REACHABLE:
                 debug.dbg('Return reachable: %s', r)
                 break
         return types
 
     def _eval_yield(self, yield_expr):
+        if yield_expr.type == 'keyword':
+            # `yield` just yields None.
+            yield context.LazyKnownContext(compiled.create(self.evaluator, None))
+            return
+
         node = yield_expr.children[1]
         if node.type == 'yield_arg':  # It must be a yield from.
             cn = ContextualizedNode(self, node.children[1])
@@ -339,9 +362,9 @@ class FunctionExecutionContext(context.TreeContext):
 
     @recursion.execution_recursion_decorator(default=iter([]))
     def get_yield_values(self):
-        for_parents = [(y, tree.search_ancestor(y, ('for_stmt', 'funcdef',
-                                                    'while_stmt', 'if_stmt')))
-                       for y in self.tree_node.yields]
+        for_parents = [(y, tree.search_ancestor(y, 'for_stmt', 'funcdef',
+                                                'while_stmt', 'if_stmt'))
+                       for y in get_yield_exprs(self.evaluator, self.tree_node)]
 
         # Calculate if the yields are placed within the same for loop.
         yields_order = []
@@ -353,7 +376,7 @@ class FunctionExecutionContext(context.TreeContext):
             if parent.type == 'suite':
                 parent = parent.parent
             if for_stmt.type == 'for_stmt' and parent == self.tree_node \
-                    and for_stmt.defines_one_name():  # Simplicity for now.
+                    and parser_utils.for_stmt_defines_one_name(for_stmt):  # Simplicity for now.
                 if for_stmt == last_for_stmt:
                     yields_order[-1][1].append(yield_)
                 else:
@@ -375,12 +398,12 @@ class FunctionExecutionContext(context.TreeContext):
                     for result in self._eval_yield(yield_):
                         yield result
             else:
-                input_node = for_stmt.get_input_node()
+                input_node = for_stmt.get_testlist()
                 cn = ContextualizedNode(self, input_node)
                 ordered = iterable.py__iter__(evaluator, cn.infer(), cn)
                 ordered = list(ordered)
                 for lazy_context in ordered:
-                    dct = {str(for_stmt.children[1]): lazy_context.infer()}
+                    dct = {str(for_stmt.children[1].value): lazy_context.infer()}
                     with helpers.predefine_names(self, for_stmt, dct):
                         for yield_in_same_for_stmt in yields:
                             for result in self._eval_yield(yield_in_same_for_stmt):
@@ -391,20 +414,9 @@ class FunctionExecutionContext(context.TreeContext):
                                              until_position=until_position,
                                              origin_scope=origin_scope)
 
-    @memoize_default(default=NO_DEFAULT)
+    @evaluator_method_cache()
     def get_params(self):
-        return param.get_params(self.evaluator, self.parent_context, self.tree_node, self.var_args)
-
-
-class AnonymousFunctionExecution(FunctionExecutionContext):
-    def __init__(self, evaluator, parent_context, function_context):
-        super(AnonymousFunctionExecution, self).__init__(
-            evaluator, parent_context, function_context, var_args=None)
-
-    @memoize_default(default=NO_DEFAULT)
-    def get_params(self):
-        # We need to do a dynamic search here.
-        return search_params(self.evaluator, self.parent_context, self.tree_node)
+        return self.var_args.get_params(self)
 
 
 class ModuleAttributeName(AbstractNameDefinition):
@@ -460,12 +472,12 @@ class ModuleContext(use_metaclass(CachedMetaClass, context.TreeContext)):
     # I'm not sure if the star import cache is really that effective anymore
     # with all the other really fast import caches. Recheck. Also we would need
     # to push the star imports into Evaluator.modules, if we reenable this.
-    @memoize_default([])
+    @evaluator_method_cache([])
     def star_imports(self):
         modules = []
-        for i in self.tree_node.imports:
+        for i in self.tree_node.iter_imports():
             if i.is_star_import():
-                name = i.star_import_name()
+                name = i.get_paths()[-1][-1]
                 new = imports.infer_import(self, name)
                 for module in new:
                     if isinstance(module, ModuleContext):
@@ -473,7 +485,7 @@ class ModuleContext(use_metaclass(CachedMetaClass, context.TreeContext)):
                 modules += new
         return modules
 
-    @memoize_default()
+    @evaluator_method_cache()
     def _module_attributes_dict(self):
         names = ['__file__', '__package__', '__doc__', '__name__']
         # All the additional module attributes are strings.
@@ -491,7 +503,7 @@ class ModuleContext(use_metaclass(CachedMetaClass, context.TreeContext)):
             return re.sub('\.[a-z]+-\d{2}[mud]{0,3}$', '', r.group(1))
 
     @property
-    @memoize_default()
+    @evaluator_method_cache()
     def name(self):
         return ModuleName(self, self._string_name)
 
@@ -535,7 +547,7 @@ class ModuleContext(use_metaclass(CachedMetaClass, context.TreeContext)):
         init_path = self.py__file__()
         if os.path.basename(init_path) == '__init__.py':
             with open(init_path, 'rb') as f:
-                content = common.source_to_unicode(f.read())
+                content = python_bytes_to_unicode(f.read(), errors='replace')
                 # these are strings that need to be used for namespace packages,
                 # the first one is ``pkgutil``, the second ``pkg_resources``.
                 options = ('declare_namespace(__name__)', 'extend_path(__path__')
@@ -574,7 +586,7 @@ class ModuleContext(use_metaclass(CachedMetaClass, context.TreeContext)):
         else:
             return self._py__path__
 
-    @memoize_default()
+    @evaluator_method_cache()
     def _sub_modules_dict(self):
         """
         Lists modules in the directory of this module (if this module is a
@@ -638,7 +650,7 @@ class ImplicitNamespaceContext(use_metaclass(CachedMetaClass, context.TreeContex
         yield DictFilter(self._sub_modules_dict())
 
     @property
-    @memoize_default()
+    @evaluator_method_cache()
     def name(self):
         string_name = self.py__package__().rpartition('.')[-1]
         return ImplicitNSName(self, string_name)
@@ -655,7 +667,7 @@ class ImplicitNamespaceContext(use_metaclass(CachedMetaClass, context.TreeContex
     def py__path__(self):
         return lambda: [self.paths]
 
-    @memoize_default()
+    @evaluator_method_cache()
     def _sub_modules_dict(self):
         names = {}
 
